@@ -34,6 +34,9 @@ do
 	if [ -x test.sh ]; then
 		./test.sh
 		if [ $? -ne 0 ]; then
+			if [[ $TEST_LOG =~ dump ]]; then
+				cat "$RESULTS"
+			fi
 			_fail "test failed for case $(basename $i)"
 		fi
 	fi
